@@ -1,9 +1,9 @@
 /*! \file aciacore.c \n
- *  \author Andr Fachat, Spiro Trikaliotis\n
+ *  \author Andre Fachat, Spiro Trikaliotis\n
  *  \brief  Template file for ACIA 6551 emulation.
  *
  * Written by
- *  Andr Fachat <fachat@physik.tu-chemnitz.de>
+ *  Andre Fachat <fachat@physik.tu-chemnitz.de>
  *  Spiro Trikaliotis <spiro.trikaliotis@gmx.de>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
@@ -34,23 +34,26 @@
 
 #include "acia.h"
 #include "alarm.h"
-#include "clkguard.h"
 #include "cmdline.h"
 #include "interrupt.h"
 #include "log.h"
 #include "machine.h"
 #include "resources.h"
 #include "rs232drv.h"
+#include "rs232.h"
 #include "snapshot.h"
-#include "translate.h"
 #include "types.h"
+#include "monitor.h"
 
+uint8_t myacia_read(uint16_t addr);
 
 #undef  DEBUG   /*!< define if you want "normal" debugging output */
 #undef  DEBUG_VERBOSE /*!< define if you want very verbose debugging output. */
 /* #define DEBUG */
 /* #define DEBUG_VERBOSE */
 
+/* #define LOG_MODEM_STATUS */
+
 /*! \brief Helper macro for outputting debugging messages */
 #ifdef DEBUG
 # define DEBUG_LOG_MESSAGE(_x) log_message _x
@@ -84,22 +87,22 @@ typedef struct acia_struct {
     unsigned int int_num;     /*!< the (internal) number for the ACIA interrupt as returned by interrupt_cpu_status_int_new(). */
 
     int ticks;  /*!< number of clock ticks per char */
-    int ticks_rx;  /*!< number of clock ticks per char for reception (workaround for NovaTerm, cf. remark for set_acia_ticks() */
     int fd;             /*!< file descriptor used to access the RS232 physical device on the host machine */
     enum acia_tx_state in_tx;   /*!< indicates that a transmit is currently ongoing */
     int irq;
-    BYTE cmd;        /*!< value of the 6551 command register */
-    BYTE ctrl;       /*!< value of the 6551 control register */
-    BYTE rxdata;     /*!< data that has been received last */
-    BYTE txdata;     /*!< data prepared to be send */
-    BYTE status;     /*!< value of the 6551 status register */
-    BYTE ectrl;      /*!< value of the extended control register of the turbo232 card */
+    uint8_t cmd;        /*!< value of the 6551 command register */
+    uint8_t ctrl;       /*!< value of the 6551 control register */
+    uint8_t rxdata;     /*!< data that has been received last */
+    uint8_t txdata;     /*!< data prepared to be send */
+    uint8_t status;     /*!< value of the 6551 status register */
+    uint8_t ectrl;      /*!< value of the extended control register of the turbo232 card */
+    uint8_t datamask;   /*!< Word length bitmask used on received and sent data */
     int alarm_active_tx;    /*!< 1 if TX alarm is set; else 0 */
     int alarm_active_rx;    /*!< 1 if RX alarm is set; else 0 */
 
     log_t log; /*!< the log where to write debugging messages */
 
-    BYTE last_read;  /*!< the byte read the last time (for RMW) */
+    uint8_t last_read;  /*!< the byte read the last time (for RMW) */
 
     /******************************************************************/
 
@@ -156,22 +159,23 @@ typedef struct acia_struct {
 
     /*! \brief The handshake lines as currently seen by the ACIA */
     enum rs232handshake_out rs232_status_lines;
+
 } acia_type;
 
-
-
 /******************************************************************/
 
-static acia_type acia = { NULL };
+static acia_type acia = { NULL, NULL, 0, 0, 0, (enum acia_tx_state)0,
+                          0, 0, 0, 0, 0, 0, 0, 0xff, 0, 0, 0, 0, 0, 0, 0,
+                          (enum cpu_int)0, 0, 0, (enum rs232handshake_out)0 };
 
-void acia_preinit(void)
+static void acia_preinit(void)
 {
     memset(&acia, 0, sizeof acia);
 
-    acia.ticks = acia.ticks_rx = 21111;
+    acia.ticks = 21111;
     acia.fd = -1;
     acia.in_tx = ACIA_TX_STATE_NO_TRANSMIT;
-    acia.log = LOG_ERR;
+    acia.log = LOG_DEFAULT;
     acia.irq_type = IK_NONE;
     acia.mode = ACIA_MODE_NORMAL;
 }
@@ -249,6 +253,7 @@ static int acia_set_device(int val, void *param)
     return 0;
 }
 
+
 /*! \internal \brief Generate an ACIA interrupt
 
  This function is used to generate an ACIA interrupt.
@@ -275,6 +280,8 @@ static int acia_set_device(int val, void *param)
 */
 static void acia_set_int(int aciairq, unsigned int int_num, int value)
 {
+    DEBUG_LOG_MESSAGE((acia.log, "acia_set_int(aciairq=%d, int_num=%u, value=%u",
+        aciairq, int_num, value));
     assert((value == aciairq) || (value == IK_NONE));
 
     if (aciairq == IK_IRQ) {
@@ -290,8 +297,8 @@ static void acia_set_int(int aciairq, unsigned int int_num, int value)
  \param new_irq_res
    The interrupt type to use:
       0 = none,
-      1 = IRQ,
-      2 = NMI.
+      1 = NMI,
+      2 = IRQ.
 
  \param param
    Unused
@@ -307,19 +314,14 @@ static void acia_set_int(int aciairq, unsigned int int_num, int value)
 static int acia_set_irq(int new_irq_res, void *param)
 {
     enum cpu_int new_irq;
-    static const enum cpu_int irq_tab[] = { IK_NONE, IK_IRQ, IK_NMI };
+    static const enum cpu_int irq_tab[] = { IK_NONE, IK_NMI, IK_IRQ };
 
     /*
      * if an invalid interrupt type has been given, return
      * with an error.
      */
-    switch (new_irq_res) {
-        case IK_NONE:
-        case IK_NMI:
-        case IK_IRQ:
-            break;
-        default:
-            return -1;
+    if ((new_irq_res < 0) || (new_irq_res > 2)) {
+        return -1;
     }
 
     new_irq = irq_tab[new_irq_res];
@@ -359,7 +361,7 @@ static double get_acia_bps(void)
             }
 
         default:
-            log_message(acia.log, "Invalid acia.mode = %u in get_acia_bps()", acia.mode);
+            log_message(acia.log, "Invalid acia.mode = %d in get_acia_bps()", acia.mode);
             return acia_bps_table[0]; /* return dummy value */
     }
 }
@@ -397,8 +399,14 @@ static void set_acia_ticks(void)
             break;
     }
 
+    /* set the data bitmask */
+
+    acia.datamask = 0xff >> (8 - bits);
+
     /*
      * we neglect the fact that we might have 1.5 stop bits instead of 2
+     *
+     * FIX ME!!! 8-bit mode with parity should have only 1 stop bit
      */
     bits += 1 /* the start bit */
             + ((acia.cmd & ACIA_CMD_BITS_PARITY_ENABLED) ? 1 : 0) /* parity or not */
@@ -410,20 +418,10 @@ static void set_acia_ticks(void)
      */
     acia.ticks = (int) (machine_get_cycles_per_second() / get_acia_bps() * bits);
 
-    /*
-     * Note: With 10 bit, the timing is very hard to NovaTerm 9.6c with 57600 bps
-     * on reception. It cannot cope and behaves erroneously, at least when configured
-     * for NMI interrupts. This is because the interrupt routine needs too much
-     * time to execute, leaving not more than 20 cycles for the non-NMI routine.
-     * Thus, it was decided to add 25% "safety margin" to allow NovaTerm to react
-     * appropriately. This gives the main program 50 extra cycles.
-     * This way, NovaTerm can cope with the transmission at 57600 bps.
-     */
-    acia.ticks_rx = acia.ticks * 5 / 4;
 
     /* adjust the alarm rate for reception */
     if (acia.alarm_active_rx) {
-        acia.alarm_clk_rx = myclk + acia.ticks_rx;
+        acia.alarm_clk_rx = myclk + acia.ticks;
         alarm_set(acia.alarm_rx, acia.alarm_clk_rx);
         acia.alarm_active_rx = 1;
     }
@@ -431,7 +429,9 @@ static void set_acia_ticks(void)
     /*
      * set the baud rate of the physical device
      */
-    rs232drv_set_bps(acia.fd, (unsigned int)get_acia_bps());
+    if (acia.fd >= 0) {
+        rs232drv_set_bps(acia.fd, (unsigned int)get_acia_bps());
+    }
 }
 
 /*! \internal \brief Change the emulation mode for this ACIA
@@ -473,7 +473,7 @@ static int acia_set_mode(int new_mode, void *param)
 static const resource_int_t resources_int[] = {
     { MYACIA "Dev", MyDevice, RES_EVENT_NO, NULL,
       &acia.device, acia_set_device, NULL },
-    { NULL }
+    RESOURCE_INT_LIST_END
 };
 
 /*! \brief initialize the ACIA resources
@@ -489,19 +489,19 @@ int myacia_init_resources(void)
     acia_preinit();
 
     acia.irq_res = MyIrq;
+    acia.irq_type = MyIrq;
     acia.mode = ACIA_MODE_NORMAL;
 
     return resources_register_int(resources_int);
 }
 
 /*! \brief the command-line options available for the ACIA */
-static const cmdline_option_t cmdline_options[] = {
-    { "-myaciadev", SET_RESOURCE, 1,
+static const cmdline_option_t cmdline_options[] =
+{
+    { "-myaciadev", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, MYACIA "Dev", NULL,
-      USE_PARAM_STRING, USE_DESCRIPTION_ID,
-      IDCLS_UNUSED, IDCLS_SPECIFY_ACIA_RS232_DEVICE,
-      "<0-3>", NULL },
-    { NULL }
+      "<0-3>", "Specify RS232 device this ACIA should work on" },
+    CMDLINE_LIST_END
 };
 
 /*! \brief initialize the command-line options
@@ -520,30 +520,6 @@ int myacia_init_cmdline_options(void)
 /******************************************************************/
 /* auxiliary functions */
 
-/*! \internal \brief Prevent clock overflow by adjusting clock value
-
- \param sub
-   The number of clock ticks to adjust the clock by subtracting
-   from the current value
-
- \param var
-   The data as has been given to clk_guard_add_callback() as
-   3rd parameter. For this implementation, always NULL.
-
- \remark
-   In order to prevent a clock overflow, the system is able
-   to subtract a given amount from the clock values. When this
-   happens, this function is called in order for the module to
-   adjust its own values.
-*/
-static void clk_overflow_callback(CLOCK sub, void *var)
-{
-    assert(var == NULL);
-
-    acia.alarm_clk_tx -= sub;
-    acia.alarm_clk_rx -= sub;
-}
-
 /*! \internal \brief Get the modem status and set the status register accordingly
 
  This function reads the physical modem status lines (DSR, DCD)
@@ -551,13 +527,21 @@ static void clk_overflow_callback(CLOCK sub, void *var)
 
  \return
    The new value of the status register
+
+ \todo
+   Changes in DSR and DCD should trigger an interrupt
 */
 static int acia_get_status(void)
 {
-    enum rs232handshake_in modem_status = rs232drv_get_status(acia.fd);
+    enum rs232handshake_in modem_status = 0;
+#ifdef LOG_MODEM_STATUS
+    static int oldstatus = -1;
+#endif
 
+    if (acia.fd >= 0) {
+        modem_status = rs232drv_get_status(acia.fd);
+    }
     acia.status &= ~(ACIA_SR_BITS_DCD | ACIA_SR_BITS_DSR);
-
 #if 0
     /*
      * CTS is very different from DCD.
@@ -568,11 +552,39 @@ static int acia_get_status(void)
         acia.status |= ACIA_SR_BITS_DCD; /* we treat CTS like DCD */
     }
 #endif
+    if (!(modem_status & RS232_HSI_DCD)) {
+        /* DCD mirrors DSR for C64/128 machines */
+        switch (machine_class) {
+            case VICE_MACHINE_C64:      /* fall through */
+            case VICE_MACHINE_C64SC:    /* fall through */
+            case VICE_MACHINE_SCPU64:   /* fall through */
+            case VICE_MACHINE_C128:     /* fall through */
+                acia.status |= ACIA_SR_BITS_DSR;
+            break;
+        /* Real DCD for other machines */
+            default:
+                acia.status |= ACIA_SR_BITS_DCD;
+        }
+    }
 
-    if (modem_status & RS232_HSI_DSR) {
+    if (!(modem_status & RS232_HSI_DSR)) {
         acia.status |= ACIA_SR_BITS_DSR;
     }
 
+
+#ifdef LOG_MODEM_STATUS
+    if (acia.status != oldstatus) {
+        printf("acia_get_status(fd:%d): modem_status:%02x dcd:%d dsr:%d status:%02x dcd:%d dsr:%d\n",
+               acia.fd, modem_status,
+               modem_status & RS232_HSI_DCD ? 1 : 0,
+               modem_status & RS232_HSI_DSR ? 1 : 0,
+               acia.status,
+               acia.status & ACIA_SR_BITS_DCD ? 1 : 0,
+               acia.status & ACIA_SR_BITS_DSR ? 1 : 0
+        );
+        oldstatus = acia.status;
+    }
+#endif
     return acia.status;
 }
 
@@ -583,14 +595,21 @@ static int acia_get_status(void)
 */
 static void acia_set_handshake_lines(void)
 {
+#ifdef LOG_MODEM_STATUS
+    static int oldstatus = -1;
+#endif
     switch (acia.cmd & ACIA_CMD_BITS_TRANSMITTER_MASK) {
         case ACIA_CMD_BITS_TRANSMITTER_NO_RTS:
             /* unset RTS, we are NOT ready to receive */
             acia.rs232_status_lines &= ~RS232_HSO_RTS;
             if (acia.alarm_active_rx) {
-                /* diable RX alarm */
+                /* disable RX alarm */
+                /* receiver gets disabled after current character is completed */
                 acia.alarm_active_rx = 0;
-                alarm_unset(acia.alarm_rx);
+                /* disable and unset TX alarm */
+                /* transmitter is disabled immediately */
+                acia.alarm_active_tx = 0;
+                alarm_unset(acia.alarm_tx);
             }
             break;
 
@@ -607,6 +626,12 @@ static void acia_set_handshake_lines(void)
                 acia.alarm_active_rx = 1;
                 set_acia_ticks();
             }
+            /* start tx alarm */
+            if (acia.alarm_active_tx == 0) {
+                    acia.alarm_clk_tx = myclk + acia.ticks;
+                    alarm_set(acia.alarm_tx, acia.alarm_clk_tx);
+                    acia.alarm_active_tx = 1;
+            }
             break;
     }
 
@@ -617,8 +642,22 @@ static void acia_set_handshake_lines(void)
         /* unset DTR, we are NOT ready to receive or to transmit */
         acia.rs232_status_lines &= ~RS232_HSO_DTR;
     }
+
+#ifdef LOG_MODEM_STATUS
+    if (acia.rs232_status_lines != oldstatus) {
+        printf("acia_set_handshake_lines(fd:%d): rs232 status:%02x dtr:%d rts:%d\n",
+               acia.fd,
+               acia.rs232_status_lines,
+               acia.rs232_status_lines & RS232_HSO_DTR ? 1 : 0,
+               acia.rs232_status_lines & RS232_HSO_RTS ? 1 : 0
+        );
+        oldstatus = acia.rs232_status_lines;
+    }
+#endif
     /* set the RTS and the DTR status */
-    rs232drv_set_status(acia.fd, acia.rs232_status_lines);
+    if (acia.fd >= 0) {
+        rs232drv_set_status(acia.fd, acia.rs232_status_lines);
+    }
 }
 
 /*! \brief initialize the ACIA */
@@ -629,9 +668,7 @@ void myacia_init(void)
     acia.alarm_tx = alarm_new(mycpu_alarm_context, MYACIA, int_acia_tx, NULL);
     acia.alarm_rx = alarm_new(mycpu_alarm_context, MYACIA, int_acia_rx, NULL);
 
-    clk_guard_add_callback(mycpu_clk_guard, clk_overflow_callback, NULL);
-
-    if (acia.log == LOG_ERR) {
+    if (acia.log == LOG_DEFAULT) {
         acia.log = log_open(MYACIA);
     }
 }
@@ -642,7 +679,9 @@ void myacia_reset(void)
     DEBUG_LOG_MESSAGE((acia.log, "reset_myacia"));
 
     acia.rs232_status_lines = 0;
-    rs232drv_set_status(acia.fd, acia.rs232_status_lines);
+    if (acia.fd >= 0) {
+        rs232drv_set_status(acia.fd, acia.rs232_status_lines);
+    }
 
     acia.cmd = ACIA_CMD_DEFAULT_AFTER_HW_RESET;
     acia.ctrl = ACIA_CTRL_DEFAULT_AFTER_HW_RESET;
@@ -650,7 +689,10 @@ void myacia_reset(void)
 
     set_acia_ticks();
 
-    acia.status = ACIA_SR_DEFAULT_AFTER_HW_RESET;
+    /* ACIA Status Register after HW reset = 0xx10000 */
+    acia.status &= ACIA_SR_BITS_DSR;    /* Keep DSR from emulated modem */
+    acia.status |= ACIA_SR_BITS_DCD | ACIA_SR_DEFAULT_AFTER_HW_RESET; /* But disable DCD, closing the rs232drv will bring it up anyways */
+
     acia.in_tx = ACIA_TX_STATE_NO_TRANSMIT;
 
     if (acia.fd >= 0) {
@@ -678,11 +720,11 @@ void myacia_reset(void)
 /* The dump format has a module header and the data generated by the
  * chip...
  *
- * The version of this dump description is 0/0
+ * The version of this dump description is 1.0
  */
 
 #define ACIA_DUMP_VER_MAJOR      1 /*!< the major version number of the dump data */
-#define ACIA_DUMP_VER_MINOR      0 /*!< the minor version number of the dump data */
+#define ACIA_DUMP_VER_MINOR      1 /*!< the minor version number of the dump data */
 
 /*
  * Layout of the dump data:
@@ -695,9 +737,9 @@ void myacia_reset(void)
  *
  * UBYTE        IN_TX   0 = no data to tx; 2 = TDR valid; 1 = in transmit (cf. enum acia_tx_state)
  *
- * DWORD        TICKSTX ticks till the next TDR empty interrupt
+ * QWORD        TICKSTX ticks till the next TDR empty interrupt
  *
- * DWORD        TICKSRX ticks till the next RDF empty interrupt
+ * QWORD        TICKSRX ticks till the next RDF empty interrupt
  *                      TICKSRX has been added with 2.0.9; if it does not
  *                      exist on read, it is assumed that it has the same
  *                      value as TICKSTX to emulate the old behaviour.
@@ -726,37 +768,42 @@ static const char module_name[] = MYACIA;
 int myacia_snapshot_write_module(snapshot_t *p)
 {
     snapshot_module_t *m;
+    CLOCK act;
+    CLOCK aar;
+
+    m = snapshot_module_create(p, module_name, ACIA_DUMP_VER_MAJOR, ACIA_DUMP_VER_MINOR);
 
-    m = snapshot_module_create(p, module_name, (BYTE)ACIA_DUMP_VER_MAJOR,
-                               (BYTE)ACIA_DUMP_VER_MINOR);
     if (m == NULL) {
         return -1;
     }
 
-    SMW_B(m, acia.txdata);
-    SMW_B(m, acia.rxdata);
-    SMW_B(m, (BYTE)(acia_get_status() | (acia.irq ? ACIA_SR_BITS_IRQ : 0)));
-    SMW_B(m, acia.cmd);
-    SMW_B(m, acia.ctrl);
-    SMW_B(m, (BYTE)(acia.in_tx));
-
     if (acia.alarm_active_tx) {
-        SMW_DW(m, (acia.alarm_clk_tx - myclk));
+        act = acia.alarm_clk_tx - myclk;
     } else {
-        SMW_DW(m, 0);
+        act = 0;
     }
 
-    /* new with VICE 2.0.9 */
-
     if (acia.alarm_active_rx) {
-        SMW_DW(m, (acia.alarm_clk_rx - myclk));
+        aar = acia.alarm_clk_rx - myclk;
     } else {
-        SMW_DW(m, 0);
+        aar = 0;
     }
 
-    snapshot_module_close(m);
+    if (SMW_B(m, acia.txdata) < 0
+            || SMW_B(m, acia.rxdata) < 0
+            || SMW_B(m, (uint8_t)(acia_get_status()
+                    | (acia.irq ? ACIA_SR_BITS_IRQ : 0))) < 0
+            || SMW_B(m, acia.cmd) < 0
+            || SMW_B(m, acia.ctrl) < 0
+            || SMW_B(m, (uint8_t)(acia.in_tx)) < 0
+            || SMW_CLOCK(m, act) < 0
+            || SMW_CLOCK(m, aar) < 0) {
+        snapshot_module_close(m);
+        return -1;
+    }
 
-    return 0;
+
+    return snapshot_module_close(m);
 }
 
 /*! \brief Read the snapshot module for the ACIA
@@ -781,9 +828,10 @@ int myacia_snapshot_write_module(snapshot_t *p)
 */
 int myacia_snapshot_read_module(snapshot_t *p)
 {
-    BYTE vmajor, vminor;
-    BYTE byte;
-    DWORD dword;
+    uint8_t vmajor, vminor;
+    uint8_t byte;
+    CLOCK qword1;
+    CLOCK qword2;
     snapshot_module_t *m;
 
     alarm_unset(acia.alarm_tx);   /* just in case we don't find module */
@@ -794,20 +842,30 @@ int myacia_snapshot_read_module(snapshot_t *p)
     mycpu_set_int_noclk(acia.int_num, 0);
 
     m = snapshot_module_open(p, module_name, &vmajor, &vminor);
+
     if (m == NULL) {
         return -1;
     }
 
-    if (vmajor != ACIA_DUMP_VER_MAJOR) {
+    /* Do not accept versions higher than current */
+    if (snapshot_version_is_bigger(vmajor, vminor, ACIA_DUMP_VER_MAJOR, ACIA_DUMP_VER_MINOR)) {
+        snapshot_set_error(SNAPSHOT_MODULE_HIGHER_VERSION);
         snapshot_module_close(m);
         return -1;
     }
 
-    SMR_B(m, &acia.txdata);
-    SMR_B(m, &acia.rxdata);
+    if (SMR_B(m, &acia.txdata) < 0
+            || SMR_B(m, &acia.rxdata) < 0
+            || SMR_B(m, &acia.status) < 0
+            || SMR_B(m, &acia.cmd) < 0
+            || SMR_B(m, &acia.ctrl) < 0
+            || SMR_B(m, &byte) < 0
+            || SMR_CLOCK(m, &qword1) < 0) {
+        snapshot_module_close(m);
+        return -1;
+    }
 
     acia.irq = 0;
-    SMR_B(m, &acia.status);
     if (acia.status & ACIA_SR_BITS_IRQ) {
         acia.status &= ~ACIA_SR_BITS_IRQ;
         acia.irq = 1;
@@ -816,26 +874,22 @@ int myacia_snapshot_read_module(snapshot_t *p)
         mycpu_set_int_noclk(acia.int_num, 0);
     }
 
-    SMR_B(m, &acia.cmd);
     if ((acia.cmd & ACIA_CMD_BITS_DTR_ENABLE_RECV_AND_IRQ) && (acia.fd < 0)) {
         acia.fd = rs232drv_open(acia.device);
         acia_set_handshake_lines();
     } else {
-        if ((acia.fd >= 0) && !(acia.cmd & ACIA_CMD_BITS_DTR_ENABLE_RECV_AND_IRQ)) {
+        if ((acia.fd >= 0) && !(acia.cmd & ACIA_CMD_BITS_DTR_ENABLE_RECV_AND_IRQ) && !rs232_useip232[acia.device]) {
             rs232drv_close(acia.fd);
             acia.fd = -1;
         }
     }
 
-    SMR_B(m, &acia.ctrl);
     set_acia_ticks();
 
-    SMR_B(m, &byte);
     acia.in_tx = byte;
 
-    SMR_DW(m, &dword);
-    if (dword) {
-        acia.alarm_clk_tx = myclk + dword;
+    if (qword1) {
+        acia.alarm_clk_tx = myclk + qword1;
         alarm_set(acia.alarm_tx, acia.alarm_clk_tx);
         acia.alarm_active_tx = 1;
 
@@ -845,7 +899,7 @@ int myacia_snapshot_read_module(snapshot_t *p)
          * if we have a new snapshot (2.0.9 and up), this will be
          * overwritten directly afterwards.
          */
-        acia.alarm_clk_rx = myclk + dword;
+        acia.alarm_clk_rx = myclk + qword1;
         alarm_set(acia.alarm_rx, acia.alarm_clk_rx);
         acia.alarm_active_rx = 1;
     }
@@ -854,9 +908,9 @@ int myacia_snapshot_read_module(snapshot_t *p)
      * this is new with VICE 2.0.9; thus, only use the settings
      * if it does exist.
      */
-    if (SMR_DW(m, &dword) >= 0) {
-        if (dword) {
-            acia.alarm_clk_rx = myclk + dword;
+    if (SMR_CLOCK(m, &qword2) >= 0) {
+        if (qword2) {
+            acia.alarm_clk_rx = myclk + qword2;
             alarm_set(acia.alarm_rx, acia.alarm_clk_rx);
             acia.alarm_active_rx = 1;
         } else {
@@ -865,11 +919,7 @@ int myacia_snapshot_read_module(snapshot_t *p)
         }
     }
 
-    if (snapshot_module_close(m) < 0) {
-        return -1;
-    }
-
-    return 0;
+    return snapshot_module_close(m);
 }
 
 
@@ -882,7 +932,7 @@ int myacia_snapshot_read_module(snapshot_t *p)
   \param byte
     The value to set the register to
 */
-void myacia_store(WORD addr, BYTE byte)
+void myacia_store(uint16_t addr, uint8_t byte)
 {
     int acia_register_size;
 
@@ -909,6 +959,7 @@ void myacia_store(WORD addr, BYTE byte)
                     log_message(acia.log, "ACIA: data register written "
                                 "although data has not been sent yet.");
                 }
+                DEBUG_LOG_MESSAGE((acia.log, "DR write at %d: 0x%02x", myclk, acia.txdata));
                 acia.in_tx = ACIA_TX_STATE_DR_WRITTEN;
                 if (acia.alarm_active_tx == 0) {
                     acia.alarm_clk_tx = myclk + 1;
@@ -921,13 +972,18 @@ void myacia_store(WORD addr, BYTE byte)
         case ACIA_SR:
             /* According the CSG and WDC data sheets, this is a programmed reset! */
 
-            if (acia.fd >= 0) {
+            if ((acia.fd >= 0) && !rs232_useip232[acia.device]) {
                 rs232drv_close(acia.fd);
+                acia.fd = -1;
             }
-            acia.fd = -1;
 
+            /* Status Register programmed reset = xxxxx0xx */
             acia.status &= ~ACIA_SR_BITS_OVERRUN_ERROR;
+            /* Command Register programmed reset = xxx00000 */
             acia.cmd &= ACIA_CMD_BITS_PARITY_TYPE_MASK | ACIA_CMD_BITS_PARITY_ENABLED;
+            /* This bit is set only in the MOS 6551, not the Rockwell 6551 or the 65c51 versions */
+            /* acia.cmd |= ACIA_CMD_BITS_IRQ_DISABLED; */
+
             acia.in_tx = ACIA_TX_STATE_NO_TRANSMIT;
             acia_set_int(acia.irq_type, acia.int_num, IK_NONE);
             acia.irq = 0;
@@ -943,19 +999,27 @@ void myacia_store(WORD addr, BYTE byte)
             break;
         case ACIA_CMD:
             acia.cmd = byte;
-            acia_set_handshake_lines();
             if ((acia.cmd & ACIA_CMD_BITS_DTR_ENABLE_RECV_AND_IRQ) && (acia.fd < 0)) {
                 acia.fd = rs232drv_open(acia.device);
                 /* enable RX alarm */
                 acia.alarm_active_rx = 1;
                 set_acia_ticks();
+                /* Set Tx alarm if Tx IRQs are enabled */
+                if ((acia.cmd & ACIA_CMD_BITS_TRANSMITTER_MASK) == ACIA_CMD_BITS_TRANSMITTER_TX_WITH_IRQ) {
+                    acia.alarm_clk_tx = myclk + acia.ticks;
+                    alarm_set(acia.alarm_tx, acia.alarm_clk_tx);
+                    acia.alarm_active_tx = 1;
+                    acia.in_tx = ACIA_TX_STATE_NO_TRANSMIT;
+                }
             } else
-            if ((acia.fd >= 0) && !(acia.cmd & ACIA_CMD_BITS_DTR_ENABLE_RECV_AND_IRQ)) {
+            if ((acia.fd >= 0) && !(acia.cmd & ACIA_CMD_BITS_DTR_ENABLE_RECV_AND_IRQ) && !rs232_useip232[acia.device]) {
                 rs232drv_close(acia.fd);
                 alarm_unset(acia.alarm_tx);
                 acia.alarm_active_tx = 0;
                 acia.fd = -1;
             }
+            /* Moved here so rs232drv status is always updated */
+            acia_set_handshake_lines();
             break;
         case T232_ECTRL:
             if ((acia.ctrl & ACIA_CTRL_BITS_BPS_MASK) == ACIA_CTRL_BITS_BPS_16X_EXT_CLK) {
@@ -976,13 +1040,13 @@ void myacia_store(WORD addr, BYTE byte)
   \return
     The value the register has
 */
-BYTE myacia_read(WORD addr)
+uint8_t myacia_read(uint16_t addr)
 {
 #if 0 /* def DEBUG */
-    static BYTE myacia_read_(WORD);
-    BYTE byte = myacia_read_(addr);
-    static WORD last_addr = 0;
-    static BYTE last_byte = 0;
+    static uint8_t myacia_read_(uint16_t);
+    uint8_t byte = myacia_read_(addr);
+    static uint16_t last_addr = 0;
+    static uint8_t last_byte = 0;
 
     if ((addr != last_addr) || (byte != last_byte)) {
         DEBUG_LOG_MESSAGE((acia.log, "read_myacia(%04x) -> %02x", addr, byte));
@@ -990,7 +1054,7 @@ BYTE myacia_read(WORD addr)
     last_addr = addr; last_byte = byte;
     return byte;
 }
-static BYTE myacia_read_(WORD addr)
+static uint8_t myacia_read_(uint16_t addr)
 {
 #endif
     int acia_register_size;
@@ -1003,12 +1067,14 @@ static BYTE myacia_read_(WORD addr)
 
     switch (addr & acia_register_size) {
         case ACIA_DR:
-            acia.status &= ~ACIA_SR_BITS_RECEIVE_DR_FULL;
+            DEBUG_LOG_MESSAGE((acia.log, "DR read at %d: 0x%02x", myclk, acia.rxdata));
+            acia.status &= ~(ACIA_SR_BITS_OVERRUN_ERROR | ACIA_SR_BITS_PARITY_ERROR | ACIA_SR_BITS_FRAMING_ERROR | ACIA_SR_BITS_RECEIVE_DR_FULL);
             acia.last_read = acia.rxdata;
             return acia.rxdata;
         case ACIA_SR:
             {
-                BYTE c = acia_get_status() | (acia.irq ? ACIA_SR_BITS_IRQ : 0);
+                uint8_t c = acia_get_status() | (acia.irq ? ACIA_SR_BITS_IRQ : 0);
+                DEBUG_LOG_MESSAGE((acia.log, "SR read at %d: 0x%02x", myclk,c));
                 acia_set_int(acia.irq_type, acia.int_num, IK_NONE);
                 acia.irq = 0;
                 acia.last_read = c;
@@ -1044,18 +1110,15 @@ static BYTE myacia_read_(WORD addr)
 
   \return
     The value the register has
-
-  \todo
-    Currently unused
 */
-BYTE myacia_peek(WORD addr)
+uint8_t myacia_peek(uint16_t addr)
 {
     switch (addr & 3) {
         case ACIA_DR:
             return acia.rxdata;
         case ACIA_SR:
             {
-                BYTE c = acia.status | (acia.irq ? ACIA_SR_BITS_IRQ : 0);
+                uint8_t c = acia.status | (acia.irq ? ACIA_SR_BITS_IRQ : 0);
                 return c;
             }
         case ACIA_CTRL:
@@ -1095,9 +1158,6 @@ BYTE myacia_peek(WORD addr)
    ensure that we do not send out faster than a real ACIA could
    do.
 
- \todo
-   If no transmit is in progress (in_tx == ACIA_TX_STATE_NO_TRANSMIT),
-   it is not necessary to schedule a new alarm.
 */
 static void int_acia_tx(CLOCK offset, void *data)
 {
@@ -1106,16 +1166,17 @@ static void int_acia_tx(CLOCK offset, void *data)
     assert(data == NULL);
 
     if ((acia.in_tx == ACIA_TX_STATE_DR_WRITTEN) && (acia.fd >= 0)) {
-        rs232drv_putc(acia.fd, acia.txdata);
+        rs232drv_putc(acia.fd, acia.txdata & acia.datamask);
 
         /* tell the status register that the transmit register is empty */
         acia.status |= ACIA_SR_BITS_TRANSMIT_DR_EMPTY;
+    }
 
-        /* generate an interrupt if the ACIA was programmed to generate one */
-        if ((acia.cmd & ACIA_CMD_BITS_TRANSMITTER_MASK) == ACIA_CMD_BITS_TRANSMITTER_TX_WITH_IRQ) {
-            acia_set_int(acia.irq_type, acia.int_num, acia.irq_type);
-            acia.irq = 1;
-        }
+    /* generate an interrupt if the ACIA was programmed to generate one */
+    /* interrupt will occur everytime even if no data was transmitted */
+    if ((acia.cmd & ACIA_CMD_BITS_TRANSMITTER_MASK) == ACIA_CMD_BITS_TRANSMITTER_TX_WITH_IRQ) {
+        acia_set_int(acia.irq_type, acia.int_num, acia.irq_type);
+        acia.irq = 1;
     }
 
     if (acia.in_tx != ACIA_TX_STATE_NO_TRANSMIT) {
@@ -1126,7 +1187,7 @@ static void int_acia_tx(CLOCK offset, void *data)
         acia.in_tx--;
     }
 
-    if (acia.in_tx != ACIA_TX_STATE_NO_TRANSMIT) {
+    if ((acia.in_tx != ACIA_TX_STATE_NO_TRANSMIT) || ((acia.cmd & ACIA_CMD_BITS_TRANSMITTER_MASK) == ACIA_CMD_BITS_TRANSMITTER_TX_WITH_IRQ)) {
         /* re-schedule alarm */
         acia.alarm_clk_tx = myclk + acia.ticks;
         alarm_set(acia.alarm_tx, acia.alarm_clk_tx);
@@ -1168,7 +1229,7 @@ static void int_acia_rx(CLOCK offset, void *data)
     assert(data == NULL);
 
     do {
-        BYTE received_byte;
+        uint8_t received_byte;
 
         if (acia.fd < 0) {
             break;
@@ -1181,10 +1242,15 @@ static void int_acia_rx(CLOCK offset, void *data)
         DEBUG_LOG_MESSAGE((acia.log, "received byte: %u = '%c'.",
                            (unsigned) received_byte, received_byte));
 
-        /*! \todo: What happens on the real 6551? Is the old value overwritten in
-         * case of an overrun, or is it not?
-         */
-        acia.rxdata = received_byte;
+        /* Datasheet (https://downloads.reactivemicro.com/Electronics/Interface%20Adapters/R65C51.pdf)
+         * says that new data is discarded on overrun */
+        if (!(acia.status & ACIA_SR_BITS_RECEIVE_DR_FULL)) {
+            acia.rxdata = received_byte & acia.datamask;
+        } else {
+            acia.status |= ACIA_SR_BITS_OVERRUN_ERROR;
+            DEBUG_LOG_MESSAGE((acia.log, "Overrun! Discarding received byte",
+                           (unsigned) acia.rxdata, acia.rxdata));
+        }
 
         /* generate an interrupt if the ACIA was configured to generate one */
         if (!(acia.cmd & ACIA_CMD_BITS_IRQ_DISABLED)) {
@@ -1192,22 +1258,65 @@ static void int_acia_rx(CLOCK offset, void *data)
             acia.irq = 1;
         }
 
-        if (acia.status & ACIA_SR_BITS_RECEIVE_DR_FULL) {
-            acia.status |= ACIA_SR_BITS_OVERRUN_ERROR;
-            break;
-        }
 
         acia.status |= ACIA_SR_BITS_RECEIVE_DR_FULL;
     } while (0);
 
-
-    acia.alarm_clk_rx = myclk + acia.ticks_rx;
-    alarm_set(acia.alarm_rx, acia.alarm_clk_rx);
-    acia.alarm_active_rx = 1;
+    if (acia.alarm_active_rx == 1) {
+        acia.alarm_clk_rx = myclk + acia.ticks;
+        alarm_set(acia.alarm_rx, acia.alarm_clk_rx);
+        /*acia.alarm_active_rx = 1;*/
+    } else {
+        alarm_unset(acia.alarm_rx);
+    }
 }
 
-int acia_dump(void *acia_context)
+int acia_dump(void)
 {
-    /* FIXME: dump details using mon_out(), return 0 on success */
-    return -1;
+    uint8_t st;
+    uint8_t wl;
+    char* sb;
+    const char* parity = "NONENMNS";
+    char p;
+
+    /* Status register */
+    st = myacia_peek(0x01);
+    /* Word length */
+    wl = 8 - ((myacia_peek(0x03) & 0x60) >> 5);
+    /* Parity bit */
+    p = parity[(myacia_peek(0x02) & 0xE0) >> 5];
+    /* Stop bit(s) */
+    if (myacia_peek(0x03) & 0x80) {
+        switch (wl) {
+            case 8:
+                if (p != 'N') {
+                    sb = "1";
+                } else {
+                    sb = "2";
+                }
+                break;
+            case 5:
+                if (p == 'N') {
+                    sb = "1.5";
+                } else {
+                    sb = "2";
+                }
+                break;
+            default:
+                sb = "2";
+        }
+    } else {
+        sb = "1";
+    }
+
+    mon_out("Receive Interrupt: %s\n", (myacia_peek(0x02) & 0x02) ? "off" : "on");
+    mon_out("DR Rx: %02x Status: %s\t%s\t%s\t%s\n", myacia_peek(0x00), (st & 0x08) ? "[Full]" : "[Not Full]", (st & 0x01) ? "[Parity Error]" : "",
+        (st & 0x02) ? "[Framming Error]" : "", (st & 0x04) ? "[Overrun]" : "");
+    mon_out("\nTransmit Interrupt: %s\n", ((myacia_peek(0x02) & 0x0c) == 0x04) ? "on" : "off");
+    mon_out("DR Tx: %02x Status: %s\n", acia.txdata, (st & 0x10) ? "[Empty]" : "[Not Empty]");
+    mon_out("\nRTS: %s\tDTR: %s\n", (myacia_peek(0x02) & 0x0c) ? "Low" : "High", (myacia_peek(0x02) & 0x01) ? "Low" : "High");
+    mon_out("DCD: %s\tDSR: %s\n", (st & 0x20) ? "High" : "Low", (st & 0x40) ? "High" : "Low");
+    mon_out("\nSpeed/format: %g bps / %u-%c-%s\n", get_acia_bps(), wl, p, sb);
+    mon_out("Echo: %s\n", (myacia_peek(0x02) & 0x10) ? "On" : "Off");
+    return 0;
 }
