@@ -31,10 +31,13 @@
 #include "types.h"
 #include "uimenu.h"
 
+UI_MENU_CALLBACK(MIDIInDev_dynmenu_callback);
+UI_MENU_CALLBACK(MIDIOutDev_dynmenu_callback);
+
 extern const ui_menu_entry_t midi_c64_menu[];
 extern const ui_menu_entry_t midi_vic20_menu[];
 
-extern void sdl_menu_midi_in_free(void);
-extern void sdl_menu_midi_out_free(void);
+void sdl_menu_midi_in_free(void);
+void sdl_menu_midi_out_free(void);
 
 #endif
