@@ -2,7 +2,8 @@
  * sdlmain.c - SDL startup.
  *
  * Written by
- *  Andreas Boose <viceteam@t-online.de>
+ *  Hannu Nuotio <hannu.nuotio@tut.fi>
+ *  Marco van den Heuvel <blackystardust68@yahoo.com>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -28,34 +29,41 @@
 
 #include <stdio.h>
 
+#include "cmdline.h"
 #include "log.h"
 #include "machine.h"
 #include "main.h"
+#include "resources.h"
+#include "uimenu.h"
 
 #include "vice_sdl.h"
 
-#ifdef __XBOX__
-void XBoxStartup(void)
-{
-    int argc = 1;
-    char *argv[2];
+/* FIXME: Ugly hack for preventing SDL crash using -help */
+int sdl_help_shutdown = 0;
 
-    argv[0] = "vice";
-    argv[1] = NULL;
-
-    main_program(argc, argv);
-}
-#else
 int main(int argc, char **argv)
 {
     return main_program(argc, argv);
 }
-#endif
 
 void main_exit(void)
 {
+    /* FIXME: Ugly hack for preventing SDL crash using -help */
+    if (!sdl_help_shutdown) {
+        /* log resources with non default values */
+        resources_log_active();
+        /* log the active config as commandline options */
+        cmdline_log_active();
+    }
+
     log_message(LOG_DEFAULT, "\nExiting...");
 
+    /*
+     * Clean up dangling resources due to the 'Quit emu' callback not returning
+     * to the calling menu code.
+     */
+    sdl_ui_menu_shutdown();
+
     machine_shutdown();
 
     putchar('\n');
