@@ -3,9 +3,10 @@
  * ($DC00).
  *
  * Written by
- *  Andr Fachat <fachat@physik.tu-chemnitz.de>
+ *  Andre Fachat <fachat@physik.tu-chemnitz.de>
  *  Ettore Perazzoli <ettore@comm2000.it>
  *  Andreas Boose <viceteam@t-online.de>
+ *  Marco van den Heuvel <blackystardust68@yahoo.com>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -34,9 +35,12 @@
 #include "c64-resources.h"
 #include "c64.h"
 #include "c64cia.h"
+#include "c64iec.h"
+#include "c64keyboard.h"
 #include "cia.h"
 #include "interrupt.h"
-#include "drivecpu.h"
+#include "drive.h"
+#include "joyport.h"
 #include "joystick.h"
 #include "keyboard.h"
 #include "lib.h"
@@ -44,38 +48,39 @@
 #include "machine.h"
 #include "maincpu.h"
 #include "types.h"
-#include "userport_joystick.h"
+#include "userport.h"
 #include "vicii.h"
 
-#ifdef HAVE_RS232
-#include "rsuser.h"
-#endif
-
-#ifdef HAVE_MOUSE
-#include "mouse.h"
-#endif
-
 /* #define DEBUG_KBD */
 
 #ifdef DEBUG_KBD
+#define DBG(x) printf x
 #define DBGA(x) printf x
 #define DBGB(x) printf x
 #else
+#define DBG(x)
 #define DBGA(x)
 #define DBGB(x)
 #endif
 
-void cia1_store(WORD addr, BYTE data)
+static uint8_t cia1_cra = 0;    /* last value written to control register A */
+
+void cia1_store(uint16_t addr, uint8_t data)
 {
+    if ((addr & 0xf) == CIA_CRA) {
+        cia1_cra = data;
+        DBG(("cia1_store cra: %02x\n", data));
+    }
+
     ciacore_store(machine_context.cia1, addr, data);
 }
 
-BYTE cia1_read(WORD addr)
+uint8_t cia1_read(uint16_t addr)
 {
     return ciacore_read(machine_context.cia1, addr);
 }
 
-BYTE cia1_peek(WORD addr)
+uint8_t cia1_peek(uint16_t addr)
 {
     return ciacore_peek(machine_context.cia1, addr);
 }
@@ -101,7 +106,7 @@ static void cia_restore_int(cia_context_t *cia_context, int value)
  * I/O
  */
 
-void cia1_set_extended_keyboard_rows_mask(BYTE value)
+void cia1_set_extended_keyboard_rows_mask(uint8_t value)
 {
 }
 
@@ -128,20 +133,22 @@ static void do_reset_cia(cia_context_t *cia_context)
 {
 }
 
-static void cia1_internal_lightpen_check(BYTE pa, BYTE pb)
+static void cia1_internal_lightpen_check(uint8_t pa, uint8_t pb)
 {
-    BYTE val = 0xff;
-    BYTE msk = pa & ~joystick_value[2];
-    BYTE m;
+    uint8_t val = 0xff;
+    uint8_t msk = pa & read_joyport_dig(JOYPORT_2);
+    uint8_t m;
     int i;
 
-    for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
-        if (!(msk & m)) {
-            val &= ~keyarr[i];
+    if (c64keyboard_active) {
+        for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+            if (!(msk & m)) {
+                val &= ~keyarr[i];
+            }
         }
     }
 
-    m = val & pb & ~joystick_value[1];
+    m = val & pb & read_joyport_dig(JOYPORT_1);
 
     vicii_set_light_pen(maincpu_clk, !(m & 0x10));
 }
@@ -151,43 +158,27 @@ void cia1_check_lightpen(void)
     cia1_internal_lightpen_check(machine_context.cia1->old_pa, machine_context.cia1->old_pb);
 }
 
-static void store_ciapa(cia_context_t *cia_context, CLOCK rclk, BYTE b)
+static void store_ciapa(cia_context_t *cia_context, CLOCK rclk, uint8_t b)
 {
     cia1_internal_lightpen_check(b, machine_context.cia1->old_pb);
 
-#ifdef HAVE_MOUSE
-    mouse_set_input((b >> 6) & 0x03);
+    set_joyport_pot_mask((b >> 6) & 3);
 
-    if (_mouse_enabled && (mouse_port == 2)) {
-        if (mouse_type == MOUSE_TYPE_NEOS) {
-            neos_mouse_store(b);
-        } else if (mouse_type == MOUSE_TYPE_SMART) {
-            smart_mouse_store(b);
-        }
-    }
-#endif
+    store_joyport_dig(JOYPORT_2, b, 0xff);
 }
 
-static void undump_ciapa(cia_context_t *cia_context, CLOCK rclk, BYTE b)
+static void undump_ciapa(cia_context_t *cia_context, CLOCK rclk, uint8_t b)
 {
 }
 
-static void store_ciapb(cia_context_t *cia_context, CLOCK rclk, BYTE byte)
+static void store_ciapb(cia_context_t *cia_context, CLOCK rclk, uint8_t byte)
 {
     cia1_internal_lightpen_check(machine_context.cia1->old_pa, byte);
 
-#ifdef HAVE_MOUSE
-    if (_mouse_enabled && (mouse_port == 1)) {
-        if (mouse_type == MOUSE_TYPE_NEOS) {
-            neos_mouse_store(byte);
-        } else if (mouse_type == MOUSE_TYPE_SMART) {
-            smart_mouse_store(byte);
-        }
-    }
-#endif
+    store_joyport_dig(JOYPORT_1, byte, 0xff);
 }
 
-static void undump_ciapb(cia_context_t *cia_context, CLOCK rclk, BYTE byte)
+static void undump_ciapb(cia_context_t *cia_context, CLOCK rclk, uint8_t byte)
 {
 }
 
@@ -204,18 +195,21 @@ static void undump_ciapb(cia_context_t *cia_context, CLOCK rclk, BYTE byte)
          the toplevel keyboard emulation later, so it can be used by all machines
          with a similar keyboard matrix. (VIC20, C16 ...)
  */
-static void matrix_activate_column(int column, BYTE *activerows, BYTE *activecolumns);
-static void matrix_activate_row(int row, BYTE *activerows, BYTE *activecolumns);
+static void matrix_activate_column(int column, uint8_t *activerows, uint8_t *activecolumns);
+static void matrix_activate_row(int row, uint8_t *activerows, uint8_t *activecolumns);
 
-static void matrix_activate_row(int row, BYTE *activerows, BYTE *activecolumns)
+static void matrix_activate_row(int row, uint8_t *activerows, uint8_t *activecolumns)
 {
-    BYTE msk;
+    uint8_t msk;
     int m, i;
+
     if ((1 << row) & ~(*activerows)) {
         *activerows |= (1 << row);
         msk = keyarr[row];
+
         /* loop over columns */
         for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+
             /* activate each column connected to the given row */
             if ((msk & m) & ~(*activecolumns)) {
                 matrix_activate_column(i, activerows, activecolumns);
@@ -224,15 +218,18 @@ static void matrix_activate_row(int row, BYTE *activerows, BYTE *activecolumns)
     }
 }
 
-static void matrix_activate_column(int column, BYTE *activerows, BYTE *activecolumns)
+static void matrix_activate_column(int column, uint8_t *activerows, uint8_t *activecolumns)
 {
-    BYTE msk;
+    uint8_t msk;
     int m, i;
+
     if ((1 << column) & ~(*activecolumns)) {
         *activecolumns |= (1 << column);
         msk = rev_keyarr[column];
+
         /* loop over rows */
         for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+
             /* activate each row connected to the given column */
             if ((msk & m) & ~(*activerows)) {
                 matrix_activate_row(i, activerows, activecolumns);
@@ -242,37 +239,37 @@ static void matrix_activate_column(int column, BYTE *activerows, BYTE *activecol
 }
 
 /* get all connected rows for one active column */
-inline static BYTE matrix_get_active_rows_by_column(int column)
+inline static uint8_t matrix_get_active_rows_by_column(int column)
 {
-    BYTE activerows = 0;
-    BYTE activecolumns = 0;
+    uint8_t activerows = 0;
+    uint8_t activecolumns = 0;
     matrix_activate_column(column, &activerows, &activecolumns);
     return activerows;
 }
 
 /* get all connected rows for one active row */
-inline static BYTE matrix_get_active_rows_by_row(int row)
+inline static uint8_t matrix_get_active_rows_by_row(int row)
 {
-    BYTE activerows = 0;
-    BYTE activecolumns = 0;
+    uint8_t activerows = 0;
+    uint8_t activecolumns = 0;
     matrix_activate_row(row, &activerows, &activecolumns);
     return activerows;
 }
 
 /* get all connected columns for one active row */
-inline static BYTE matrix_get_active_columns_by_column(int column)
+inline static uint8_t matrix_get_active_columns_by_column(int column)
 {
-    BYTE activerows = 0;
-    BYTE activecolumns = 0;
+    uint8_t activerows = 0;
+    uint8_t activecolumns = 0;
     matrix_activate_column(column, &activerows, &activecolumns);
     return activecolumns;
 }
 
 /* get all connected columns for one active row */
-inline static BYTE matrix_get_active_columns_by_row(int row)
+inline static uint8_t matrix_get_active_columns_by_row(int row)
 {
-    BYTE activerows = 0;
-    BYTE activecolumns = 0;
+    uint8_t activerows = 0;
+    uint8_t activecolumns = 0;
     matrix_activate_row(row, &activerows, &activecolumns);
     return activecolumns;
 }
@@ -286,12 +283,12 @@ inline static BYTE matrix_get_active_columns_by_row(int row)
     - add improvements also to C128
 */
 
-static BYTE read_ciapa(cia_context_t *cia_context)
+static uint8_t read_ciapa(cia_context_t *cia_context)
 {
-    BYTE byte;
-    BYTE val = 0xff;
-    BYTE msk;
-    BYTE m, tmp;
+    uint8_t byte;
+    uint8_t val = 0xff;
+    uint8_t msk;
+    uint8_t m, tmp;
     int i;
 
     DBGA(("PA ddra:%02x pa:%02x ddrb:%02x pb:%02x ",
@@ -301,21 +298,23 @@ static BYTE read_ciapa(cia_context_t *cia_context)
     /* loop over columns,
        pull down all bits connected to a column which is output and active.
      */
-    msk = cia_context->old_pb & ~joystick_value[1];
-    for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
-        if (!(msk & m)) {
-            tmp = matrix_get_active_columns_by_column(i);
+    msk = cia_context->old_pb & read_joyport_dig(JOYPORT_1);
+    if (c64keyboard_active) {
+        for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+            if (!(msk & m)) {
+                tmp = matrix_get_active_columns_by_column(i);
 
-            /* when scanning from port B to port A with inactive bits set to 1
-               in port B, ghostkeys will be eliminated (pulled high) if the
-               matrix is connected to more 1 bits of port B. this does NOT happen
-               when the respective bits are set to input. (see testprogs/CIA/ciaports)
-             */
-            if (tmp & cia_context->c_cia[CIA_PRB] & cia_context->c_cia[CIA_DDRB]) {
-                val &= ~rev_keyarr[i];
-                DBGA(("<force high %02x>", m));
-            } else {
-                val &= ~matrix_get_active_rows_by_column(i);
+                /* when scanning from port B to port A with inactive bits set to 1
+                   in port B, ghostkeys will be eliminated (pulled high) if the
+                   matrix is connected to more 1 bits of port B. this does NOT happen
+                   when the respective bits are set to input. (see testprogs/CIA/ciaports)
+                 */
+                if (tmp & cia_context->c_cia[CIA_PRB] & cia_context->c_cia[CIA_DDRB]) {
+                    val &= ~rev_keyarr[i];
+                    DBGA(("<force high %02x>", m));
+                } else {
+                    val &= ~matrix_get_active_rows_by_column(i);
+                }
             }
         }
     }
@@ -325,61 +324,51 @@ static BYTE read_ciapa(cia_context_t *cia_context)
        pull down all bits connected to a row which is output and active.
        handles the case when port a is used for both input and output
      */
-    msk = cia_context->old_pa & ~joystick_value[2];
-    for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
-        if (!(msk & m)) {
-            val &= ~matrix_get_active_rows_by_row(i);
+    msk = cia_context->old_pa & read_joyport_dig(JOYPORT_2);
+    if (c64keyboard_active) {
+        for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+            if (!(msk & m)) {
+                val &= ~matrix_get_active_rows_by_row(i);
+            }
         }
     }
     DBGA((" val:%02x", val));
 
-    byte = (val & (cia_context->c_cia[CIA_PRA] | ~(cia_context->c_cia[CIA_DDRA]))) & ~joystick_value[2];
+    byte = (val & (cia_context->c_cia[CIA_PRA] | ~(cia_context->c_cia[CIA_DDRA]))) & read_joyport_dig(JOYPORT_2);
 
     DBGA((" out:%02x\n", byte));
 
-#ifdef HAVE_MOUSE
-    if (_mouse_enabled && (mouse_port == 2)) {
-        if (mouse_type == MOUSE_TYPE_NEOS) {
-            byte &= neos_mouse_read();
-        } else if (mouse_type == MOUSE_TYPE_SMART) {
-            byte &= smart_mouse_read();
-        } else if (mouse_kind == MOUSE_KIND_POLLED) {
-            byte &= mouse_poll();
-        }
-    }
-#endif
-
     return byte;
 }
 
-inline static int ciapb_forcelow(int row, BYTE mask)
+inline static int ciapb_forcelow(int row, uint8_t mask)
 {
-    BYTE v;
+    uint8_t v;
 
-    /* Check for shift lock.
-       FIXME: keyboard_shiftlock state may be inconsistent
-              with the (rev_)keyarr state. */
-    if ((row == 1) && keyboard_shiftlock) {
-        return 1;
-    }
+    if (c64keyboard_active) {
+        /* Check for shift lock. */
+        if ((row == 1) && keyboard_get_shiftlock()) {
+            return 1;
+        }
 
-    /* Check if two or more rows are connected */
-    v = matrix_get_active_rows_by_row(row) & mask;
-    if ((v & (v - 1)) != 0) {
-        return 1;
+        /* Check if two or more rows are connected */
+        v = matrix_get_active_rows_by_row(row) & mask;
+        if ((v & (v - 1)) != 0) {
+            return 1;
+        }
     }
 
     /* TODO: check joysticks? */
     return 0;
 }
 
-static BYTE read_ciapb(cia_context_t *cia_context)
+static uint8_t read_ciapb(cia_context_t *cia_context)
 {
-    BYTE byte;
-    BYTE val = 0xff;
-    BYTE val_outhi;
-    BYTE msk, tmp;
-    BYTE m;
+    uint8_t byte;
+    uint8_t val = 0xff;
+    uint8_t val_outhi;
+    uint8_t msk, tmp;
+    uint8_t m;
     int i;
 
     /* loop over rows,
@@ -391,26 +380,28 @@ static BYTE read_ciapb(cia_context_t *cia_context)
           cia_context->c_cia[CIA_DDRA], cia_context->c_cia[CIA_PRA],
           cia_context->c_cia[CIA_DDRB], cia_context->c_cia[CIA_PRB]));
 
-    msk = cia_context->old_pa & ~joystick_value[2];
-    for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
-        if (!(msk & m)) {
-            tmp = matrix_get_active_columns_by_row(i);
-            val &= ~tmp;
+    msk = cia_context->old_pa & read_joyport_dig(JOYPORT_2);
+    if (c64keyboard_active) {
+        for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+            if (!(msk & m)) {
+                tmp = matrix_get_active_columns_by_row(i);
+                val &= ~tmp;
 
-            /*
-                Handle the special case when both port A and port B are programmed as output,
-                port A outputs (active) low, and port B outputs high.
+                /*
+                    Handle the special case when both port A and port B are programmed as output,
+                    port A outputs (active) low, and port B outputs high.
 
-                In this case either connecting one port A 0 bit (by pressing either shift-lock)
-                or two or more port A 0 bits (by pressing keys of the same column) to one port B
-                bit is required to drive port B low (see testprogs/CIA/ciaports)
-            */
-            if ((cia_context->c_cia[CIA_DDRA] & ~cia_context->c_cia[CIA_PRA] & m) &&
-                (cia_context->c_cia[CIA_DDRB] & cia_context->c_cia[CIA_PRB] & tmp)) {
-                DBGB(("(%d)", i));
-                if (ciapb_forcelow(i, (BYTE)(cia_context->c_cia[CIA_DDRA] & ~cia_context->c_cia[CIA_PRA]))) {
-                    val_outhi &= ~tmp;
-                    DBGB(("<force low, val_outhi:%02x>", val_outhi));
+                    In this case either connecting one port A 0 bit (by pressing either shift-lock)
+                    or two or more port A 0 bits (by pressing keys of the same column) to one port B
+                    bit is required to drive port B low (see testprogs/CIA/ciaports)
+                */
+                if ((cia_context->c_cia[CIA_DDRA] & ~cia_context->c_cia[CIA_PRA] & m) &&
+                    (cia_context->c_cia[CIA_DDRB] & cia_context->c_cia[CIA_PRB] & tmp)) {
+                    DBGB(("(%d)", i));
+                    if (ciapb_forcelow(i, (uint8_t)(cia_context->c_cia[CIA_DDRA] & ~cia_context->c_cia[CIA_PRA]))) {
+                        val_outhi &= ~tmp;
+                        DBGB(("<force low, val_outhi:%02x>", val_outhi));
+                    }
                 }
             }
         }
@@ -421,74 +412,68 @@ static BYTE read_ciapb(cia_context_t *cia_context)
        pull down all bits connected to a column which is output and active.
        handles the case when port b is used for both input and output
      */
-    msk = cia_context->old_pb & ~joystick_value[1];
-    for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
-        if (!(msk & m)) {
-            val &= ~matrix_get_active_columns_by_column(i);
+    msk = cia_context->old_pb & read_joyport_dig(JOYPORT_1);
+    DBGB((" oldpb: %02x msk: %02x", cia_context->old_pb, msk));
+    if (c64keyboard_active) {
+        for (m = 0x1, i = 0; i < 8; m <<= 1, i++) {
+            if (!(msk & m)) {
+                val &= ~matrix_get_active_columns_by_column(i);
+            }
         }
     }
+
     DBGB((" val:%02x", val));
 
     byte = val & (cia_context->c_cia[CIA_PRB] | ~(cia_context->c_cia[CIA_DDRB]));
     byte |= val_outhi;
-    byte &= ~joystick_value[1];
+    byte &= read_joyport_dig(JOYPORT_1);
 
     DBGB((" out:%02x\n", byte));
 
-#ifdef HAVE_MOUSE
-    if (_mouse_enabled && (mouse_port == 1)) {
-        if (mouse_type == MOUSE_TYPE_NEOS) {
-            byte &= neos_mouse_read();
-        } else if (mouse_type == MOUSE_TYPE_SMART) {
-            byte &= smart_mouse_read();
-        } else if (mouse_kind == MOUSE_KIND_POLLED) {
-            byte &= mouse_poll();
-        }
-    }
-#endif
-
     return byte;
 }
 
 static void read_ciaicr(cia_context_t *cia_context)
 {
     if (burst_mod == BURST_MOD_CIA1) {
-        drivecpu_execute_all(maincpu_clk);
+        drive_cpu_execute_all(maincpu_clk);
     }
 }
 
 static void read_sdr(cia_context_t *cia_context)
 {
     if (burst_mod == BURST_MOD_CIA1) {
-        drivecpu_execute_all(maincpu_clk);
+        drive_cpu_execute_all(maincpu_clk);
     }
+    cia_context->c_cia[CIA_SDR] = read_userport_sp1(cia_context->c_cia[CIA_SDR]);
+    DBG(("read_sdr sp1: %02x\n", cia_context->c_cia[CIA_SDR]));
 }
 
-static void store_sdr(cia_context_t *cia_context, BYTE byte)
+static void store_sdr(cia_context_t *cia_context, uint8_t byte)
 {
-    if (burst_mod == BURST_MOD_CIA1) {
-        c64fastiec_fast_cpu_write((BYTE)byte);
+    if ((cia1_cra & 0x49) == 0x41) {
+        DBG(("store_sdr sp1: %02x\n", byte));
+        store_userport_sp1(byte);
     }
-#ifdef HAVE_RS232
-    if (rsuser_enabled) {
-        rsuser_tx_byte(byte);
+
+    if (c64iec_active) {
+        if (burst_mod == BURST_MOD_CIA1) {
+            c64fastiec_fast_cpu_write((uint8_t)byte);
+        }
     }
-#endif
-    /* FIXME: in the upcoming userport system this call needs to be conditional */
-    userport_joystick_store_sdr(byte);
 }
 
 void cia1_init(cia_context_t *cia_context)
 {
-    ciacore_init(machine_context.cia1, maincpu_alarm_context, maincpu_int_status, maincpu_clk_guard);
+    ciacore_init(machine_context.cia1, maincpu_alarm_context, maincpu_int_status);
 }
 
-void cia1_setup_context(machine_context_t *machine_context)
+void cia1_setup_context(machine_context_t *machinecontext)
 {
     cia_context_t *cia;
 
-    machine_context->cia1 = lib_calloc(1, sizeof(cia_context_t));
-    cia = machine_context->cia1;
+    machinecontext->cia1 = lib_calloc(1, sizeof(cia_context_t));
+    cia = machinecontext->cia1;
 
     cia->prv = NULL;
     cia->context = NULL;
@@ -496,7 +481,7 @@ void cia1_setup_context(machine_context_t *machine_context)
     cia->rmw_flag = &maincpu_rmw_flag;
     cia->clk_ptr = &maincpu_clk;
 
-    cia->todticks = C64_PAL_CYCLES_PER_RFSH;
+    cia1_set_timing(cia, C64_PAL_CYCLES_PER_SEC, 50);
 
     ciacore_setup_context(cia);
 
@@ -529,7 +514,11 @@ void cia1_setup_context(machine_context_t *machine_context)
     cia->pre_peek = pre_peek;
 }
 
-void cia1_set_timing(cia_context_t *cia_context, int todticks)
+void cia1_set_timing(cia_context_t *cia_context, int tickspersec, int powerfreq)
 {
-    cia_context->todticks = todticks;
+    cia_context->power_freq = powerfreq;
+    cia_context->ticks_per_sec = tickspersec;
+    cia_context->todticks = tickspersec / powerfreq;
+    cia_context->power_tickcounter = 0;
+    cia_context->power_ticks = 0;
 }
