@@ -5,7 +5,7 @@
  *  Andreas Boose <viceteam@t-online.de>
  *  Daniel Sladic <sladic@eecg.toronto.edu>
  *  Ettore Perazzoli <ettore@comm2000.it>
- *  Andr Fachat <fachat@physik.tu-chemnitz.de>
+ *  Andre Fachat <fachat@physik.tu-chemnitz.de>
  *  Teemu Rantanen <tvr@cs.hut.fi>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
@@ -31,6 +31,11 @@
 #ifndef VICE_C64IEC_H
 #define VICE_C64IEC_H
 
-extern void c64iec_init(void);
+void c64iec_init(void);
+void c64iec_enable(int val);
+
+extern int c64iec_active;
+
+int c64iec_get_active_state(void);
 
 #endif
