@@ -29,10 +29,12 @@
 
 #include "types.h"
 
-extern void parallel_cable_cpu_execute(int type);
-extern void parallel_cable_cpu_write(int type, BYTE data);
-extern void parallel_cable_cpu_pulse(int type);
-extern BYTE parallel_cable_cpu_read(int type);
-extern void parallel_cable_cpu_undump(int type, BYTE data);
+void parallel_cable_cpu_execute(int type);
+void parallel_cable_cpu_write(int type, uint8_t data);
+void parallel_cable_cpu_pulse(int type);
+uint8_t parallel_cable_cpu_read(int type, uint8_t data);
+void parallel_cable_cpu_undump(int type, uint8_t data);
+
+int parallel_cable_cpu_resources_init(void);
 
 #endif
