@@ -31,11 +31,19 @@
 
 #include "types.h"
 
-extern void REGPARM2 comal80_io1_store(WORD addr, BYTE value);
+void comal80_config_init(void);
+void comal80_config_setup(uint8_t *rawcart);
+int comal80_bin_attach(const char *filename, uint8_t *rawcart);
+int comal80_crt_attach(FILE *fd, uint8_t *rawcart, int variant);
+void comal80_detach(void);
 
-extern void comal80_config_init(void);
-extern void comal80_config_setup(BYTE *rawcart);
-extern int comal80_crt_attach(FILE *fd, BYTE *rawcart);
-extern void comal80_detach(void);
+int comal80_cmdline_options_init(void);
+int comal80_resources_init(void);
+void comal80_resources_shutdown(void);
+
+struct snapshot_s;
+
+int comal80_snapshot_write_module(struct snapshot_s *s);
+int comal80_snapshot_read_module(struct snapshot_s *s);
 
 #endif
