@@ -24,15 +24,12 @@
  *
  */
 
-#ifndef _C64DTVMEMSNAPSHOT_H
-#define _C64DTVMEMSNAPSHOT_H
+#ifndef VICE_C64DTVMEMSNAPSHOT_H
+#define VICE_C64DTVMEMSNAPSHOT_H
 
 struct snapshot_s;
 
-extern int c64dtv_snapshot_write_module(struct snapshot_s *s, int save_roms);
-extern int c64dtv_snapshot_read_module(struct snapshot_s *s);
-extern int c64dtvmisc_snapshot_write_module(struct snapshot_s *s);
-extern int c64dtvmisc_snapshot_read_module(struct snapshot_s *s);
- 
-#endif
+int c64dtv_snapshot_write_module(struct snapshot_s *s, int save_roms);
+int c64dtv_snapshot_read_module(struct snapshot_s *s);
 
+#endif
