@@ -33,19 +33,19 @@
 
 
 int machine_bus_lib_directory(unsigned int unit, const char *pattern,
-                              BYTE **buf)
+                              uint8_t **buf)
 {
     return -1;
 }
 
 int machine_bus_lib_read_sector(unsigned int unit, unsigned int track,
-                                unsigned int sector, BYTE *buf)
+                                unsigned int sector, uint8_t *buf)
 {
     return -1;
 }
 
 int machine_bus_lib_write_sector(unsigned int unit, unsigned int track,
-                                 unsigned int sector, BYTE *buf)
+                                 unsigned int sector, uint8_t *buf)
 {
     return -1;
 }
@@ -55,7 +55,7 @@ unsigned int machine_bus_device_type_get(unsigned int unit)
     return SERIAL_DEVICE_VIRT;
 }
 
-void machine_bus_status_truedrive_set(unsigned int enable)
+void machine_bus_status_truedrive_set(unsigned int unit, unsigned int enable)
 {
 }
 
@@ -63,9 +63,8 @@ void machine_bus_status_drivetype_set(unsigned int unit, unsigned int enable)
 {
 }
 
-void machine_bus_status_virtualdevices_set(unsigned int enable)
+void machine_bus_status_trapdevices_set(unsigned int unit, unsigned int enable)
 {
-    parallel_bus_enable(enable);
 }
 
 void machine_bus_eof_callback_set(void (*func)(void))
@@ -81,4 +80,3 @@ void machine_bus_attention_callback_set(void (*func)(void))
 void machine_bus_init_machine(void)
 {
 }
-
