@@ -3,7 +3,7 @@
  *
  * Written by
  *  Ettore Perazzoli <ettore@comm2000.it>
- *  Andr Fachat <fachat@physik.tu-chemnitz.de>
+ *  Andre Fachat <fachat@physik.tu-chemnitz.de>
  *
  * 16/24bpp support added by
  *  Steven Tieu <stieu@physics.ubc.ca>
@@ -38,27 +38,35 @@
 #include "maincpu.h"
 #include "types.h"
 
+#include <string.h>
+
 
 /* CRTC interface functions.
+   - bit 0 of the addr is wired to register-select of the chip
    FIXME: Several registers are not implemented.  */
 
-void crtc_store(WORD addr, BYTE value)
+void crtc_store(uint16_t addr, uint8_t value)
 {
-    int current_cycle;
+    CLOCK current_cycle;
 
-    current_cycle = maincpu_clk - crtc.rl_start;
+    /*
+     * The current raster line starts at the left edge of the text area.
+     * That is also when the raster draw alarm is called.
+     */
+    current_cycle = maincpu_clk - CRTC_STORE_OFFSET - crtc.rl_start;
 
     addr &= 1;
 
     if (!addr) {
-        crtc.regno = value;
+        /* low on register-select permits writes to the address register */
+        crtc.regno = value & 0x1f;
         return;
     }
 
 #if 0
     /* debug display, just not the cursor (for CBM-II) */
     if (crtc.regno < 14 && crtc.regno != 10) {
-        printf("store_crtc(reg=%d, %d) - cline=%d, ycount=%d, char=%d\n",
+        printf("store_crtc(reg=%d, %d) - cline=%d, ycount=%d, char=%lld\n",
                crtc.regno, value, crtc.current_charline, crtc.raster.ycounter,
                current_cycle);
     }
@@ -67,7 +75,7 @@ void crtc_store(WORD addr, BYTE value)
     crtc.regs[crtc.regno] = value;
 
     switch (crtc.regno) {
-        case 0:                 /* R00  Horizontal total (characters + 1) */
+        case CRTC_REG_HTOTAL:       /* R00  Horizontal total (characters + 1) */
             if (current_cycle > value) {
                 value = 255;
             }
@@ -75,18 +83,19 @@ void crtc_store(WORD addr, BYTE value)
             if (crtc.initialized) {
                 alarm_set(crtc.raster_draw_alarm, crtc.rl_start + value);
             }
+            crtc.delayed_resize = true;
             break;
 
-        case 1:                 /* R01  Horizontal characters displayed */
+        case CRTC_REG_HDISP:        /* R01  Horizontal characters displayed */
             if (!(current_cycle < crtc.rl_visible)) {
                 break;
             }
 
             /* the compare is not yet done */
-            if ((crtc.regs[1]) > current_cycle) {
+            if (crtc.regs[CRTC_REG_HDISP] > current_cycle) {
                 /* only if we write a higher value than the counter,
                  we can update disp_cycles here */
-                crtc.rl_visible = crtc.regs[1];
+                crtc.rl_visible = crtc.regs[CRTC_REG_HDISP];
                 crtc.henable = 1;
             } else {
                 /* we write a value lower than the counter -> never reached,
@@ -94,39 +103,56 @@ void crtc_store(WORD addr, BYTE value)
                 crtc.rl_visible = crtc.rl_len + 1;
                 crtc.henable = 0;
             }
+#if CRTC_BEAM_RACING
+            if (crtc.venable) {
+                /* Just cache the new visible length of the line */
+                crtc_fetch_prefetch();
+            }
+#endif
+            crtc.delayed_resize = true;
             break;
 
-        case 2:                 /* R02  Horizontal Sync Position */
+        case CRTC_REG_HSYNC:        /* R02  Horizontal Sync Position */
             if (current_cycle < crtc.rl_sync) {
                 /* FIXME: middle of pulse, adjust from reg. 3 */
                 crtc.rl_sync = value;
             }
             break;
-        case 3:                 /* R03  Horizontal/Vertical Sync widths */
+
+        case CRTC_REG_SYNCWIDTH:    /* R03  Horizontal/Vertical Sync widths */
             break;
 
-        case 7:                 /* R07  Vertical sync position */
+        case CRTC_REG_VTOTAL:       /* R04  Vertical total (character) rows */
+            crtc.regs[CRTC_REG_VTOTAL] &= 0x7f;
+            crtc.delayed_resize = true;
             break;
 
-        case 8:                 /* R08  unused: Interlace and Skew */
+        case CRTC_REG_VTOTALADJ:    /* R05  Vertical total line adjust */
+            crtc.regs[CRTC_REG_VTOTALADJ] &= 0x1f;
+            crtc.delayed_resize = true;
             break;
 
-        case 6:                 /* R06  Number of display lines on screen */
-            crtc.regs[6] &= 0x7f;
+        case CRTC_REG_VDISP:        /* R06  Number of display lines on screen */
+            crtc.regs[CRTC_REG_VDISP] &= 0x7f;
+            crtc.delayed_resize = true;
             break;
 
-        case 9:                 /* R09  Rasters between two display lines */
-            crtc.regs[9] &= 0x1f;
+        case CRTC_REG_VSYNC:        /* R07  Vertical sync position */
+            crtc.regs[CRTC_REG_VSYNC] &= 0x7f;
+            crtc.delayed_resize = true;
             break;
 
-        case 4:                 /* R04  Vertical total (character) rows */
+        case CRTC_REG_MODECTRL:     /* R08  unused: Interlace and Skew */
             break;
 
-        case 5:                 /* R05  Vertical total line adjust */
+        case CRTC_REG_SCANLINE:     /* R09  number of lines per character line, including spacing */
+            crtc.regs[CRTC_REG_SCANLINE] &= 0x1f;
             break;
 
-        case 10:                /* R10  Cursor (not implemented on the PET) */
-            value = ((value >> 5) & 0x03) ^ 0x01;
+        case CRTC_REG_CURSORSTART:  /* R10  Cursor start (not implemented on the PET) */
+            crtc.regs[CRTC_REG_CURSORSTART] &= 0x7f;
+            /* FIXME: set start line */
+            value = ((value >> 5) & 0x03) ^ 0x01; /* cursor mode */
             if (!(crtc.hw_cursor && (crtc.crsrmode != value))) {
                 break;
             }
@@ -136,52 +162,62 @@ void crtc_store(WORD addr, BYTE value)
             crtc.crsrcnt = 16;
             break;
 
-        case 11:                /* R11  Cursor (not implemented on the PET) */
+        case CRTC_REG_CURSOREND:    /* R11  Cursor end (not implemented on the PET) */
+            crtc.regs[CRTC_REG_CURSOREND] &= 0x7f;
+            /* FIXME: set end line */
             break;
 
-        case 12:                /* R12  Control register */
+        case CRTC_REG_DISPSTARTH:   /* R12  Control register, MA8-13 */
             /* This is actually the upper 6 video RAM address bits.
              * But CBM decided that the two uppermost bits should be used
              * for control.
              * The usage here is from the 8032 schematics on funet.
              *
-             * Bit 0: 1=add 256 to screen start address ( 512 for 80-columns)
-             * Bit 1: 1=add 512 to screen start address (1024 for 80-columns)
-             * Bit 2: no connection
-             * Bit 3: no connection
-             * Bit 4: invert video signal
-             * Bit 5: use top half of 4K character generator
+             * Bit 0(8): 1=add 256 to screen start address ( 512 for 80-columns)
+             * Bit 1(9): 1=add 512 to screen start address (1024 for 80-columns)
+             * Bit 2(10): no connection (8296: connected)
+             * Bit 3(11): no connection (8296: connected)
+             * Bit 4(12): invert video signal
+             *            8296: HRE HiRes mode, or connected via JU8/9(?)
+             * Bit 5(13): use top half of 4K character generator
              * Bit 6: (no pin on the CRTC, video address is 14 bit only)
              * Bit 7: (no pin on the CRTC, video address is 14 bit only)
              */
+            /* FIXME: check if the above also applies to CBM2 and all the other PET models */
             /* The CRTC loads its internal counter when it starts a new
              * frame. At this point the address/mode changes are evaluated now.
              */
+            crtc.regs[CRTC_REG_DISPSTARTH] &= 0x3f;
             break;
 
-        case 13:                /* R13  Address of first character */
+        case CRTC_REG_DISPSTARTL:   /* R13  Address of first character: MA0-7 */
             break;
+
+        case CRTC_REG_CURSORPOSH:   /* R14  Cursor location  HI -- unused */
+            crtc.regs[CRTC_REG_CURSORPOSH] &= 0x3f;
 #if 0
-        case 14:
             crsr_set_dirty();
             crsrpos = ((crsrpos & 0x00ff) | ((value << 8) & 0x3f00)) & addr_mask;
             crsrrel = crsrpos - scrpos;
             crsr_set_dirty();
+#endif
             break;
 
-        case 15:                /* R14-5 Cursor location HI/LO -- unused */
+        case CRTC_REG_CURSORPOSL:   /* R15  Cursor location  LO -- unused */
+#if 0
             crsr_set_dirty();
             crsrpos = ((crsrpos & 0x3f00) | (value & 0xff)) & addr_mask;
             crsrrel = crsrpos - scrpos;
             crsr_set_dirty();
-            break;
 #endif
-        case 16:
-        case 17:                /* R16-7 Light Pen HI/LO -- read only */
+            break;
+
+        case CRTC_REG_LPENH:        /* R16 Light Pen HI -- read only */
+        case CRTC_REG_LPENL:        /* R17 Light Pen LO -- read only */
             break;
 
         case 18:
-        case 19:                /* R18-9 Update address HI/LO (only 6545)  */
+        case 19:                    /* R18-9 Update address HI/LO (only 6545)  */
             break;
 
         default:
@@ -189,7 +225,7 @@ void crtc_store(WORD addr, BYTE value)
     }
 }
 
-BYTE crtc_read(WORD addr)
+uint8_t crtc_read(uint16_t addr)
 {
     /* Status register:
      *  bit 7: 0 = register 31 (update reg.) has been read/written by CPU
@@ -202,21 +238,20 @@ BYTE crtc_read(WORD addr)
      *             timings...)
      */
     if (!(addr & 1)) {
+        /* low on register-select permits reads from the status register */
+        /* FIXME: implement bit 7 and bit 6 */
         return crtc_offscreen() ? 32 : 0;
     }
 
     /* internal registers */
     switch (crtc.regno) {
-        case 14:
-        case 15:                      /* Cursor location HI/LO */
-            if (addr >= 64) {
-                /*log_debug("crtc_read: ERROR");*/
-                return 0;
-            }
-            return crtc.regs[addr];
+        case CRTC_REG_CURSORPOSH:
+        case CRTC_REG_CURSORPOSL:
+            return crtc.regs[crtc.regno];
 
-        case 16:
-        case 17:                      /* Light Pen X,Y */
+        case CRTC_REG_LPENH:
+        case CRTC_REG_LPENL:
+            /* FIXME: Light Pen X,Y */
             return 0xff;
 
         default:
@@ -226,11 +261,17 @@ BYTE crtc_read(WORD addr)
     return 0;
 }
 
-BYTE crtc_peek(WORD addr)
+uint8_t crtc_peek(uint16_t addr)
 {
     return crtc_read(addr);
 }
 
+/* directly read a register (for monitor) */
+uint8_t crtc_peek_register(uint8_t regno)
+{
+    return crtc.regs[regno];
+}
+
 /* FIXME: to be moved to `crtc.c'.  */
 
 #if 0
