@@ -24,21 +24,20 @@
  *
  */
 
-#ifndef _PROFDOS_H
-#define _PROFDOS_H
+#ifndef VICE_PROFDOS_H
+#define VICE_PROFDOS_H
 
 #include "types.h"
 
-struct drive_context_s;
+struct diskunit_context_s;
 
-extern int profdos_cmdline_options_init(void);
+int profdos_cmdline_options_init(void);
 
-extern void profdos_init(struct drive_context_s *drv);
-extern void profdos_reset(struct drive_context_s *drv);
+void profdos_init(struct diskunit_context_s *drv);
+void profdos_reset(struct diskunit_context_s *drv);
 
-extern int profdos_load_1571(const char *name);
+int profdos_load_1571(const char *name);
 
-extern void profdos_mem_init(struct drive_context_s *drv, unsigned int type);
-
-#endif 
+void profdos_mem_init(struct diskunit_context_s *drv, unsigned int type);
 
+#endif
