@@ -42,7 +42,7 @@ static const cmdline_option_t cmdline_options[] =
       NULL, NULL, "DosName1541", NULL,
       "<Name>", "Specify name of 1541 DOS ROM image" },
     { "-dos1541II", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
-      NULL, NULL, "DosName1541II", NULL,
+      NULL, NULL, "DosName1541ii", NULL,
       "<Name>", "Specify name of 1541-II DOS ROM image" },
     { "-dos1570", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "DosName1570", NULL,
@@ -99,15 +99,17 @@ static cmdline_option_t cmd_drive[] =
       NULL, "Disable 8KiB RAM expansion at $A000-$BFFF" },
     { NULL, SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, NULL, NULL,
-      NULL, "Fixed Disk Size" },
+      "<Size>", "Fixed Disk Size" },
     CMDLINE_LIST_END
 };
 
 int iec_cmdline_options_init(void)
 {
-    unsigned int dnr, i;
+    int dnr;
 
     for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        int i;
+
         cmd_drive[0].name = lib_msprintf("-drive%iram2000", dnr + 8);
         cmd_drive[0].resource_name
             = lib_msprintf("Drive%iRAM2000", dnr + 8);
