@@ -1,12 +1,9 @@
 
-/*! \file iecbus.h 
+/*! \file iecbus.h
  *
  *  \brief IEC bus handling.
  *
  *  \author Andreas Boose <viceteam@t-online.de>
- *
- *  \page iecbus IEC bus handling
- *  \htmlinclude iec-bus.txt
  */
 
 /*
@@ -40,7 +37,7 @@
 #define IECBUS_STATUS_TRUEDRIVE      0
 #define IECBUS_STATUS_DRIVETYPE      1
 #define IECBUS_STATUS_IECDEVICE      2
-#define IECBUS_STATUS_VIRTUALDEVICES 3
+#define IECBUS_STATUS_TRAPDEVICE     3
 
 /*!
  *
@@ -61,45 +58,44 @@ typedef struct iecbus_s {
      * the drive output ports as described by the
      * IECBUS_DEVICE_WRITE_... macros
      */
-    BYTE drv_bus[IECBUS_NUM];
+    uint8_t drv_bus[IECBUS_NUM];
 
     /*! the drive output ports as seen by the drive */
-    BYTE drv_data[IECBUS_NUM];
+    uint8_t drv_data[IECBUS_NUM];
 
     /*!
      * the drive input ports, as seen by the drive
      * and also by the IECBUS_DEVICE_READ_... macros
      */
-    BYTE drv_port;
+    uint8_t drv_port;
 
     /*!
      * the computer output ports as described by the
      * IECBUS_DEVICE_WRITE_... macros
     */
-    BYTE cpu_bus;
+    uint8_t cpu_bus;
 
     /*! the computer output ports as seen by the computer */
-    BYTE cpu_port;
+    uint8_t cpu_port;
 
     /*! \todo document */
-    BYTE iec_fast_1541;
+    uint8_t iec_fast_1541;
 } iecbus_t;
 
 extern iecbus_t iecbus;
 
-extern iecbus_t *iecbus_drive_port(void);
+iecbus_t *iecbus_drive_port(void);
 
-extern void iecbus_init(void);
-extern void iecbus_cpu_undump(BYTE data);
-extern void iecbus_status_set(unsigned int type, unsigned int unit,
-                              unsigned int enable);
+void iecbus_init(void);
+void iecbus_cpu_undump(uint8_t data);
+void iecbus_status_set(unsigned int type, unsigned int unit, unsigned int enable);
 
-extern BYTE (*iecbus_callback_read)(CLOCK);
-extern void (*iecbus_callback_write)(BYTE, CLOCK);
+extern uint8_t (*iecbus_callback_read)(CLOCK);
+extern void (*iecbus_callback_write)(uint8_t, CLOCK);
+
+uint8_t iecbus_device_read(void);
+int iecbus_device_write(unsigned int unit, uint8_t data);
 
-extern BYTE iecbus_device_read(void);
-extern int  iecbus_device_write(unsigned int unit, BYTE data);
 extern void (*iecbus_update_ports)(void);
 
 #endif
-
