@@ -5,7 +5,7 @@
  *  Hannu Nuotio <hannu.nuotio@tut.fi>
  *
  * Based on code by
- *  Andr. Fachat <a.fachat@physik.tu-chemnitz.de>
+ *  Andre Fachat <a.fachat@physik.tu-chemnitz.de>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -36,19 +36,19 @@ struct midi_interface_s {
     /* Name of the interface */
     char *name;
     /* Base address (C64 specific) */
-    WORD base_addr;
+    uint16_t base_addr;
     /* Control register address */
-    WORD ctrl_addr;
+    uint16_t ctrl_addr;
     /* Status register address */
-    WORD status_addr;
+    uint16_t status_addr;
     /* Transmit register address */
-    WORD tx_addr;
+    uint16_t tx_addr;
     /* Receive register address */
-    WORD rx_addr;
+    uint16_t rx_addr;
     /* Address mask (for mirroring) */
-    WORD mask;
+    uint16_t mask;
     /* Correct counter divide for 31250 bps */
-    BYTE midi_cd;
+    uint8_t midi_cd;
     /* Interrupt type: none (0), IRQ (1) or NMI (2) */
     int irq_type;
     /* cart-ID of associated cartridge/type of expansion, 0 means internal */
@@ -56,29 +56,34 @@ struct midi_interface_s {
 };
 typedef struct midi_interface_s midi_interface_t;
 
+#define MIDI_INFERFACE_LIST_END { NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
+
 extern midi_interface_t midi_interface[];
 
 extern int midi_enabled;
 extern int midi_mode;
 
-extern void midi_init(void);
-extern void midi_reset(void);
-extern int midi_set_mode(int new_mode, void *param);
+void midi_init(void);
+void midi_reset(void);
+int midi_set_mode(int new_mode, void *param);
+
+uint8_t midi_read(uint16_t a);
+uint8_t midi_peek(uint16_t a);
+void midi_store(uint16_t a, uint8_t b);
 
-extern BYTE midi_read(WORD a);
-extern BYTE midi_peek(WORD a);
-extern void midi_store(WORD a, BYTE b);
 /* returns 1 if address is a readable MIDI register */
-extern int midi_test_read(WORD a);
-/* returns 1 if address is any MIDI register */
-extern int midi_test_peek(WORD a);
+int midi_test_read(uint16_t a);
 
-extern int midi_resources_init(void);
-extern void midi_resources_shutdown(void);
-extern int midi_cmdline_options_init(void);
+/* returns 1 if address is any MIDI register */
+int midi_test_peek(uint16_t a);
+
+int midi_resources_init(void);
+void midi_resources_shutdown(void);
+int midi_cmdline_options_init(void);
 
 struct snapshot_s;
-extern int midi_snapshot_read_module(struct snapshot_s *s);
-extern int midi_snapshot_write_module(struct snapshot_s *s);
+
+int midi_snapshot_read_module(struct snapshot_s *s);
+int midi_snapshot_write_module(struct snapshot_s *s);
 
 #endif
