@@ -111,6 +111,7 @@
 #include "userport_spt_joystick.h"
 #include "userport_synergy_joystick.h"
 #include "userport_woj_joystick.h"
+#include "userport_funmp3.h"
 #include "util.h"
 #include "via.h"
 #include "vice-event.h"
@@ -307,6 +308,10 @@ int machine_resources_init(void)
         init_resource_fail("pet");
         return -1;
     }
+    if (parallel_resources_init() < 0) {
+        init_resource_fail("parallel");
+        return -1;
+    }
     if (cartio_resources_init() < 0) {
         init_resource_fail("cartio");
         return -1;
@@ -427,6 +432,12 @@ int machine_resources_init(void)
         init_resource_fail("debug cart");
         return -1;
     }
+#if defined(USE_MPG123) && defined (HAVE_GLOB_H)
+    if (userport_funmp3_resources_init() < 0) {
+        init_resource_fail("funmp3 cart");
+        return -1;
+    }
+#endif
     return 0;
 }
 
@@ -460,6 +471,10 @@ int machine_cmdline_options_init(void)
         init_cmdline_options_fail("pet");
         return -1;
     }
+    if (parallel_cmdline_options_init() < 0) {
+        init_cmdline_options_fail("parallel");
+        return -1;
+    }
     if (cartio_cmdline_options_init() < 0) {
         init_cmdline_options_fail("cartio");
         return -1;
@@ -574,6 +589,12 @@ int machine_cmdline_options_init(void)
         init_cmdline_options_fail("debug cart");
         return -1;
     }
+#if defined(USE_MPG123) && defined (HAVE_GLOB_H)
+    if (userport_funmp3_cmdline_options_init() < 0) {
+        init_cmdline_options_fail("funmp3 cart");
+        return -1;
+    }
+#endif
     return 0;
 }
 
@@ -698,6 +719,11 @@ int machine_specific_init(void)
     /* Initialize userport based sound chips */
     userport_dac_sound_chip_init();
 
+    /* Initialize funmp3 */
+#if defined(USE_MPG123) && defined (HAVE_GLOB_H)
+    userport_funmp3_sound_chip_init();
+#endif
+
     drive_sound_init();
     datasette_sound_init();
     video_sound_init();
@@ -931,35 +957,41 @@ void pet_crtc_set_screen(void)
     /* mem_initialize_memory(); */
 
     if (!cols) {
+        /* FIXME this guesswork should never be needed!
+         * petmem_check_info() already does something like this. */
         cols = petres.rom_video;
         if (!cols) {
             cols = PET_COLS;
         }
-        vmask = (cols == 40) ? 0x3ff : 0x7ff;
+        vmask = (petres.map == PET_MAP_8296) ? 0x0fff : 0x3ff;
     }
 
-    /* when switching 8296 to 40 columns, CRTC ends up at $9000 otherwise...*/
-    if (cols == 40) {
-        vmask = 0x3ff;
-    }
 /*
     log_message(pet_mem_log, "set_screen(vmask=%04x, cols=%d, crtc=%d)",
                 vmask, cols, petres.model.crtc);
 */
     int hwflag = (cols == 80) ? CRTC_HW_DOUBLE_CHARS : 0;
+    int vrevmask;
     /* Note: see bug #1954.
      * The real cause for the timing difference is unknown so far.
      * If found, the condition below will probably change. */
     if (cols == 80 && petres.map != PET_MAP_8296) {
         hwflag |= CRTC_HW_LATE_BEAM;
     }
-    crtc_set_screen_options(cols, 25 * 10);
+    /* On the 8296 we do not (invert the screen by clearing MA12). */
+    vrevmask = petres.map == PET_MAP_8296 ? vmask : 0x1000;
+
+    /*
+     * Vertical nr of pixels: max(25*10, 256), for 25 text lines of 10 px,
+     * or 256 for the HRE.
+     */
+    crtc_set_screen_options(cols, 256);
     crtc_set_screen_addr(mem_ram + 0x8000);
     crtc_set_hw_options(hwflag,
                         vmask,
                         0x2000,         /* vchar: MA13 */
                         512,            /* vcoffset */
-                        0x1000);        /* vrevmask: MA12 */
+                        vrevmask);      /* vrevmask: MA12 */
     crtc_set_retrace_type(petres.model.crtc ? CRTC_RETRACE_TYPE_CRTC
                                             : CRTC_RETRACE_TYPE_DISCRETE);
 
