@@ -27,16 +27,34 @@
 #ifndef VICE__VERSION_H__
 #define VICE__VERSION_H__
 
+#ifndef VICE_VERSION_MAJOR
+#define VICE_VERSION_MAJOR 3
+#endif
+
+#ifndef VICE_VERSION_MINOR
+#define VICE_VERSION_MINOR 10
+#endif
+
+#ifndef VICE_VERSION_BUILD
+#define VICE_VERSION_BUILD 0
+#endif
+
 #ifndef VERSION
-#define VERSION "2.4.7"
+#define VERSION "3.10"
+#endif
+
+#ifndef VERSION_WITH_BUILD
+#define VERSION_WITH_BUILD "3.10.0"
 #endif
 
 #ifndef VERSION_RC_NUMBER
-#define VERSION_RC_NUMBER 2,4,7,0
+#define VERSION_RC_NUMBER 3,10,0,0
 #endif
 
 #ifndef PACKAGE
 #define PACKAGE "vice"
 #endif
 
+
+
 #endif
