@@ -32,12 +32,15 @@
 #define VIC20MODEL_VIC20_PAL        0
 #define VIC20MODEL_VIC20_NTSC       1
 #define VIC20MODEL_VIC21            2 /* SuperVIC (+16K) */
+#define VIC20MODEL_VIC1001          3 /* vic-1001 (japanese VIC20) */
 
-#define VIC20MODEL_NUM 3
+#define VIC20MODEL_NUM 4
 
 #define VIC20MODEL_UNKNOWN 99
 
-extern int vic20model_get(void);
-extern void vic20model_set(int model);
+#define NO_EXTRA_RAM   0
+
+int vic20model_get(void);
+void vic20model_set(int model);
 
 #endif
