@@ -30,9 +30,8 @@
 
 #include "types.h"
 
-extern void vicii_fetch_init(void);
-extern void vicii_fetch_alarm_handler(CLOCK offset, void *data);
-extern void vicii_fetch_matrix(int offs, int num, int num_0xff, int cycle);
+void vicii_fetch_init(void);
+void vicii_fetch_alarm_handler(CLOCK offset, void *data);
+void vicii_fetch_matrix(int offs, int num, int num_0xff, int cycle);
 
 #endif
-
